/*****Code to generate differential cross section tables from 2-dimensional fine histograms****/ 
/* 
   Autor: Zhihua Liang 
   last modified: Dec 11, 2011
   Email: zliang@smu.edu; liangzhihua@gmail.com
   
   read in the output ascii file and produce the cross section on each bin from the input file
   use: ./readjet <target ascii file>

*/
#include <cstdlib>
#include <cstring>
#include <fstream>
#include <string>
#include <iostream>
#include <sstream>
#include <iomanip>
#include <math.h>

using namespace std;

#define  max_ptbin 100
#define  max_ybin 10
#define  max_sample 30

//declare variable
fstream fs;
string aline;

int n_ybin;
int promode;
float siz1;
float siz2;
float ybins[max_ybin];
float n_ptbin[max_ybin];
float ptbins[max_ybin][max_ptbin];

float xsecList[max_ybin][max_ptbin];
float errorList[max_ybin][max_ptbin];
float n_datapoint[max_ybin][max_ptbin];


float interation_number[max_sample];


//declare sub program
void read_data(int, char*);
void combine(int, char**);

int main(int argc , char *argv[]){

  //pn2012
  bool NegativeXSFlag=false;
  float MaxRelErr=0.0;

  if (argc < 2)
    {
      cout << 
        "jetana: a utility for offline analysis of output histograms from MEKS\n";
      cout << "Version June 2012 by Zhihua Liang" << endl << endl;
      cout << "Usage summary\n" <<
              "============="<<endl;
      cout << 
        "1. To rebin the cross sections from MEKS histograms\n"<<
        "   sample1.dat, sample2.dat, ..., sampleN.dat \n"<<
        "   into experimental bins listed in a file bins.in, execute\n"<< 
         "    jetana sample1.dat sample2.dat ... sampleN.dat "<<endl;
      cout << "The output is written into xsec.out "<<endl << endl;
      cout << 
        "2. To combine histograms sample1.dat, sample2.dat, ..., sampleN.dat\n"<<
        "into a new histogram combine.dat, execute\n"<<
        "     jetana combine sample1.dat sample2.dat ... sampleN.dat" << endl << endl; 
      cout << "STOP: too few input parameters (" << argc-1 << ")" << endl;
      exit(3);
    }//if(argc)

  //combine the input file?
  if(!strcmp(argv[1],"combine")){
    combine(argc, argv);
    return 0;
  }

  fs.open("bins.in");
  if (!fs)
    {
      cout << "Error in opening bins.in"<<endl;
      exit(1);
    }
  
  //read header
  getline(fs,aline);
  getline(fs,aline);
  getline(fs,aline);

  // read ybins

  fs >> promode;
  fs >> n_ybin;
 
  for(int i = 0; i < n_ybin+1; ++i){
    fs>> ybins[i];
  }
  
  
  //read pt bins
  for(int i = 0; i < n_ybin; ++i){
    fs>>n_ptbin[i];
    for(int j = 0; j < n_ptbin[i]+1; ++j){
      fs>> ptbins[i][j];
    }
  }

  //init xsect and error
  for(int i = 0; i < n_ybin; ++i){
    for(int j = 0; j < n_ptbin[i]; ++j){
      xsecList[i][j] = 0.0;
      errorList[i][j] = 0.0;
    }
  }//i
  
  // read in EKS output
  for( int i = 0; i < argc - 1; ++i){
    fs.close();
    read_data(i, argv[i+1]);
  }
  
 //calculate the error and xsec
  //the sum of iteration
  float interation_sum = 0;
  for( int i = 0; i < argc - 1; ++i){
    interation_sum = interation_sum + interation_number[i] ;
  }
 
  for(int i = 0; i < n_ybin; ++i)
    for(int j = 0; j < n_ptbin[i]; ++j){
      xsecList[i][j] = xsecList[i][j]/(ptbins[i][j+1] - ptbins[i][j])/(ybins[i+1] - ybins[i])/interation_sum;
      errorList[i][j]=sqrt(errorList[i][j])/(ptbins[i][j+1] - ptbins[i][j])/(ybins[i+1] - ybins[i])/interation_sum;
  //Check if we have negative cross sections. Compute the largest relative error
      if (xsecList[i][j] < 0) NegativeXSFlag=true;
      if (xsecList[i][j]>0) 
        { double RelErr= errorList[i][j]/xsecList[i][j];
          if (RelErr > MaxRelErr) MaxRelErr=RelErr;
        }
  }//for int j

  fs.close();

 
  //print the header
  cout<<endl;
  fs.open(argv[1]);
  for(int i = 0; i < 3; ++i){
    getline(fs,aline);
    cout<<aline<<endl;
  }
  cout<<endl<<endl;
  cout<<"All energy values are in GeV and cross sections are in pb"<<endl;
  fs.close();
  //print output
  cout<<endl<<endl;
  if(promode == 1){
  cout<<"pt min"<<"\t\t"<<"pt max"<<"\t\t"<< "d(Xsec)/dp/d|y|"<<"\t\t"<<"Error"<<endl;
  }
  else if(promode == 2){
  cout<<"mas min"<<"\t\t"<<"mas max"<<"\t\t"<< "d(Xsec)/dm/dy"<<"\t\t"<<"Error"<<endl;
  }
  else if(promode == 3){
  cout<<"chi min"<<"\t\t"<<"chi max"<<"\t\t"<< "d(Xsec)/dm/dc"<<"\t\t"<<"Error"<<endl;
  }
  else{
  cout<<"v2 min"<<"\t\t"<<"v2 max"<<"\t\t"<< "d(Xsec)/dv1/dv2"<<"\t\t"<<"Error"<<endl;
  }

  for(int k = 0; k < n_ybin; ++k){
    cout.unsetf ( ios_base::floatfield  ); 
    cout<<"--------------------------------------------"<<endl;
    if(promode == 3){
    cout<<ybins[k]<<" < m < " <<ybins[k+1]<<endl;
    }
    else if(promode == 4){
    cout<<ybins[k]<<" < v1 < " <<ybins[k+1]<<endl;
    }
    else if(promode == 1){
      cout<<ybins[k]<<" < |y| < " <<ybins[k+1]<<endl;
    }
    else{
    cout<<ybins[k]<<" < y < " <<ybins[k+1]<<endl;
    }
    cout<<"--------------------------------------------"<<endl;
    for(int l = 0; l < n_ptbin[k]; ++l){
      cout<<fixed<<setprecision(2)<<ptbins[k][l]<<"\t\t"<<ptbins[k][l+1]<<"\t\t"<<scientific<<setprecision(6)<< xsecList[k][l]<<"\t\t"<< errorList[k][l]<<endl;
    }
  }
  cout<<endl;

  cout << "The largest relative error is "<<fixed<< setprecision(2)<<MaxRelErr*100.0 << " %"<<endl;
  if (NegativeXSFlag) cout << "Some cross sections are negative"<<endl;


  fs.close();

  //output to a fline
  fstream fs2("xsec.out",fstream::out);
  
  //print header
  fs2<<endl;
  fs.open(argv[1]);
  for(int i = 0; i < 4; ++i){
    getline(fs,aline);
    fs2<<aline<<endl;
  }
  fs2<<endl<<endl;
  fs2<<"all energy in GeV and Xsecs in pb"<<endl;
  fs.close();
  //print output
  fs2<<endl<<endl;
  if(promode == 1){
  fs2<<"pt min"<<"\t\t"<<"pt max"<<"\t\t"<< "d(Xsec)/dp/d|y|"<<"\t\t"<<"Error"<<endl;
  }
  else if(promode == 2){
  fs2<<"mas min"<<"\t\t"<<"mas max"<<"\t\t"<< "d(Xsec)/dm/dy"<<"\t\t"<<"Error"<<endl;
  }
  else if(promode == 3){
  fs2<<"chi min"<<"\t\t"<<"chi max"<<"\t\t"<< "d(Xsec)/dc/dm"<<"\t\t"<<"Error"<<endl;
  }
  else{
  fs2<<"v2 min"<<"\t\t"<<"v2 max"<<"\t\t"<< "d(Xsec)/dv1/dv2"<<"\t\t"<<"Error"<<endl;
  }

  for(int k = 0; k < n_ybin; ++k){
    fs2.unsetf ( ios_base::floatfield  ); 
    fs2<<"--------------------------------------------"<<endl;
    if(promode == 3){
    fs2<<ybins[k]<<" < m < " <<ybins[k+1]<<endl;
    }
    else if(promode == 4){
    fs2<<ybins[k]<<" < v1 < " <<ybins[k+1]<<endl;
    }
    else if (promode == 1){
    fs2<<ybins[k]<<" < |y| < " <<ybins[k+1]<<endl;
    }
    else {
      fs2<<ybins[k]<<" < y < " <<ybins[k+1]<<endl;
    }

    fs2<<"--------------------------------------------"<<endl;
    for(int l = 0; l < n_ptbin[k]; ++l){
      fs2<<fixed<<setprecision(2)<<ptbins[k][l]<<"\t\t"<<ptbins[k][l+1]<<"\t\t"<<scientific<<setprecision(6)<< xsecList[k][l]<<"\t\t"<< errorList[k][l]<<endl;
    }
  }
  fs2<<endl;
  
  return 0;
  }//main->


//read in data and calculate the xsec
void read_data(int i_sample, char* file_name){
  fs.open(file_name);
  
  getline(fs,aline);
  getline(fs,aline);
  getline(fs,aline);
  getline(fs,aline);
  getline(fs,aline);
  fs >> siz1 >> siz2;
  fs >> interation_number[i_sample];
  getline(fs,aline);
  
  int nn_ybin = 0;
  fs >> nn_ybin;
  
  float borders[nn_ybin][4];
  
  for(int i= 0; i < nn_ybin; ++i){
    fs>> borders[i][0] >> borders[i][1]>> borders[i][2]>> borders[i][3];
  }
  
  //calculate the cross section
  
  
  for(int i = 0; i < nn_ybin; ++i){
    float yint=borders[i][0];
    float yend=borders[i][1];
    float ptint = borders[i][2];
    float ptend=borders[i][3];
    
    float xsect = 0.0;
    float error = 0.0;
  
    for(float m = yint+siz1/2.0; m < yend; m = m + siz1){ 
      float ptcount = 0.0;
      for(float j = ptint+siz2/2.0; j < ptend; j = j + siz2){
	++ptcount;
	fs >> xsect >> error;
	
	float pt = j;
	float y = m;

        if(promode == 3){
	pt = m;
	y = j;
	}

	for(int k = 0; k < n_ybin; ++k){
	  for(int l = 0; l < n_ptbin[k]; ++l){

	    if( *(int*)&y  >= *(int*)&ybins[k] && *(int*)&y  < *(int*)&ybins[k+1] && *(int*)&pt >= *(int*)&ptbins[k][l] && *(int*)&pt < *(int*)&ptbins[k][l+1] ){
	      xsecList[k][l] =  xsecList[k][l] + xsect*interation_number[i_sample];
	      errorList[k][l] =  errorList[k][l] + error*error*interation_number[i_sample]*interation_number[i_sample];
	      ++n_datapoint[k][l];
	    }//if 
	  }//l      
	} //k
      }//j
      // cout<<setprecision(8);
      // cout<<m<<" "<<yend<<" "<<ptcount<<endl;
    }//m
  }//i
  
}// read_data

// sub program: combine all the input table file to generate an output table file which has the same format as the input one but contains the average results for the two dimensinal histogram (also add the CPU time and number of iterations in the header).
void combine(int argc, char* argv[])
{
  string aline1,aline2,aline3,aline4,aline5,aline6;
  string aline;
  int interation_number = 0;
  int seed = 0;
  float cpu_time = 0.0;
  int n_files = argc - 2;
  int nn_ybin = 0;
  string yy[20];

  //cout for the lines
  int n_bins = 0;
  fs.open(argv[2]);
  while(!fs.eof()){
    getline(fs, aline);
    n_bins++;
  }
  fs.close();
 
  double xsects[n_bins];
  double errors[n_bins];

  double xsect_avg[n_bins];
  double error_avg[n_bins];
  for(int i = 0; i < n_bins; ++i){
    xsect_avg[i] = 0.0;
    error_avg[i] = 0.0;
  }

  for(int i = 0; i < n_files; ++i){
    int inter;
    float cpu_t;
    fs.open(argv[i+2]);
    getline(fs,aline1);
    getline(fs,aline2);
    getline(fs,aline3);
    getline(fs,aline4);
    getline(fs,aline5);
    getline(fs,aline6);
    fs >> inter>>seed>>cpu_t;
    
    interation_number += inter;
    cpu_time += cpu_t;
    
    fs >> nn_ybin;
    getline(fs,aline);
    
    for(int m = 0; m < nn_ybin; ++m){
      getline(fs,yy[m]);
    }
    
    int j = 0;
    while(!fs.eof()){
      fs >> xsects[j] >> errors[j];
      xsect_avg[j] += xsects[j]*inter;
      error_avg[j] += (errors[j]*inter)*(errors[j]*inter);
      ++j;
    } //while nbins
    fs.close();
  } // nfiles
  
  n_bins = n_bins - 9 - nn_ybin;

  //average

  for(int i = 0; i < n_bins; ++i){
      xsect_avg[i] = xsect_avg[i]/(double)interation_number;
      error_avg[i] = error_avg[i]/(double)interation_number/(double)interation_number;
  }
  
  //output
  fs.open("combine.dat",fstream::out);
  fs<<aline1<<endl;
  fs<<aline2<<endl;
  fs<<aline3<<endl;
  fs<<aline4<<endl;
  fs<<aline5<<endl;
  fs<<aline6<<endl;
  fs<<"\t"<<interation_number<<"\t"<<seed<<"\t"<<cpu_time<<endl;
  fs<<"\t"<<nn_ybin<<endl;
  for(int m = 0; m < nn_ybin; ++m){
    fs<<yy[m]<<endl;
  }
  fs<<setprecision(14);
  for(int i = 0; i < n_bins; ++i){
    fs<<"      "<<xsect_avg[i]<<"        "<< sqrt(error_avg[i])<<endl;
  }
  fs.close();
}//combine->
